@@ -1,8 +1,10 @@
 import traceback
 
 from functools  import wraps
+from colored    import fg, attr
 from shutil     import rmtree
-from time   import time
+from typing     import List, Tuple
+from time   import perf_counter
 from os     import mkdir, listdir, remove
 
 def try_traceback(skip_traceback=False):
@@ -24,9 +26,9 @@ def func_timer(ret_time=False, do_print=True):
         @wraps(func)
         def wrapper(*args, **kwargs):
             fprint('Function timer started', do_print=do_print)
-            time_start = time()
+            time_start = perf_counter()
             ret_val = func(*args, **kwargs)
-            time_delta = time() - time_start
+            time_delta = perf_counter() - time_start
             fprint(f'Function {func.__name__}{args} {kwargs} Took {time_delta:.4f} seconds to run', do_print=do_print)
             if ret_time:
                 return ret_val, time_delta
@@ -35,7 +37,7 @@ def func_timer(ret_time=False, do_print=True):
     return timer
 
 
-def fprint(text=None, nl=True, flush=False, do_print=True):
+def fprint(text=None, nl=True, flush=False, do_print=True, end='') -> None:
     if not text:
         text = ''
 
@@ -43,7 +45,10 @@ def fprint(text=None, nl=True, flush=False, do_print=True):
         text = f'{text}\n'
 
     if do_print:
-        print(text, flush=flush)
+        if not end:
+            print(text, flush=flush)
+        else:
+            print(text, end=end, flush=flush)
 
 
 def finput(text='', nl=True, use_end_addon=True, ret_type: type = str):
@@ -63,10 +68,10 @@ def finput(text='', nl=True, use_end_addon=True, ret_type: type = str):
         print(f'Couldn\'t return input {inpt} as {ret_type}')
         print(f'Input type: {type(inpt)}')
         fprint('Returning value as string')
-        return inpt
+    return inpt
 
 
-def enter_to_continue(text='', nl=True, use_help_text=True):
+def enter_to_continue(text='', nl=True, use_help_text=True) -> bool:
     if text and use_help_text:
         text = f'{text} (press enter to continue) '
 
@@ -114,28 +119,53 @@ def choose_from_list(lst, header_text='', header_nl=False, input_text='Input ind
             return []
 
 
-def read_file(file_path, create=False, ret_did_create=False, remove_empty=True, do_print=True):
-    def try_read(did_create):
+def read_file(file_path, create:bool=False, ret_did_create:bool=False, remove_empty:bool=True, splitlines:bool=True, do_print:bool=True, encoding:str='utf-8', strip:bool=True) -> (Tuple[List[str] | str, bool] | List[str] | str):
+    def try_reading(did_create:bool=False) -> Tuple[List[str] | str, bool]:
+        lines: List[str] | str = 'Nothing'
         try:
-            lines = open(file_path, 'r', encoding='utf-8').read().splitlines()
+            with open(file_path, 'r', encoding=encoding) as f:
+                lines = f.read()
+            if splitlines:
+                lines = lines.splitlines()
+                if remove_empty:
+                    lines = [x for x in lines if x != '']
+                if strip:
+                    lines = [x.strip() for x in lines]
             return lines, did_create
         except FileNotFoundError:
+            fprint(f'File not found: {file_path}')
             if create:
-                create_file_dir('file', file_path, force=True)
-                did_create = True
-                return try_read(did_create)
-            else:
-                fprint(f'File not found: {file_path}', do_print=do_print) 
-    lines, did_create = try_read(False) #type:ignore
+                create_file_dir('f', file_path, force=True, do_print=do_print)
+                return try_reading(True)
+        return lines, did_create
 
-    if remove_empty and lines:
-        lines = [x for x in lines if x != '']
+    lines,did_create = try_reading()
 
     if ret_did_create:
-        return lines, did_create
+        return lines,did_create
     return lines
 
 
+def write_file(text, file_path, append=False, create=True, encoding='utf-8', do_print=True) -> bool:
+    if text.__class__.__name__ in ('list', 'tuple', 'set'):
+        text = '\n'.join(text)
+
+    lines, did_create = read_file(file_path, create=True, ret_did_create=True, remove_empty=False, do_splitlines=False, do_print=do_print)
+
+    if not did_create and lines[-1] != '\n':
+        text = '\n' + text 
+
+    if append:
+        mode = 'a'
+    else:
+        mode = 'w'
+
+    with open(file_path, mode=mode, encoding=encoding) as f:
+        f.write(text + '\n')
+
+    return did_create
+
+
 def create_file_dir(do, do_path, force=False, do_print=True):
     def create_dir():
         try:
@@ -149,10 +179,10 @@ def create_file_dir(do, do_path, force=False, do_print=True):
                 fprint(f'Directory removed: {do_path}', nl=False, do_print=do_print)
                 return create_dir()
 
-    if do == 'dir':
+    if do == 'd': # Directory
         create_dir()
     
-    elif do == 'file':
+    elif do == 'f': # File
         file_dir, filename = get_dir_path_for_file(do_path) #type:ignore
         files_in_path = listdir(file_dir)
 
@@ -168,7 +198,7 @@ def create_file_dir(do, do_path, force=False, do_print=True):
 
 @try_traceback(skip_traceback=True)
 def remove_file_dir(do, do_path, force=False, do_print=True):
-    if do == 'dir':
+    if do == 'd': # Directory
         try:
             dir_content = listdir(do_path)
         except FileNotFoundError:
@@ -182,7 +212,7 @@ def remove_file_dir(do, do_path, force=False, do_print=True):
         elif dir_content:
             fprint(f'Directory is not empty, not removing: {do_path}', do_print=do_print)
 
-    elif do == 'file':
+    elif do == 'f': # File
         lines = read_file(do_path)
         if lines and not force:
             fprint(f'File is not empty, not removing: {do_path}', do_print=do_print)
@@ -191,7 +221,7 @@ def remove_file_dir(do, do_path, force=False, do_print=True):
         fprint(f'File removed: {do_path}', do_print=do_print)
 
 
-def get_dir_path_for_file(file_path, ret_val='all'):
+def get_dir_path_for_file(file_path, ret_val='a'):
     dir_path = file_path.replace('\\', '/').split('/')
     if len(dir_path) == 1:
         dir_path = None
@@ -200,13 +230,34 @@ def get_dir_path_for_file(file_path, ret_val='all'):
         filename = dir_path.pop()
         dir_path = '/'.join(dir_path)
 
-    if ret_val == 'dir':
+    if ret_val == 'd': # Directory
         return dir_path
-    elif ret_val == 'filename':
+    elif ret_val == 'fnam': # Filename
         return filename
 
     return dir_path, filename
 
 
+def fd(variable, nl=False) -> None:
+    reset = attr('reset')
+    filename, lineno, function_name, code = traceback.extract_stack()[-2]
+    original_var = code.replace('fd(', '').replace(')', '', 1)
+    if code.replace(original_var, '') != 'fd()':
+        original_var = code
+
+    variable_type = type(variable).__name__
+
+    if variable == True:
+        variable = fg("green") + str(variable) + reset
+    elif variable == False:
+        variable = fg("red") + str(variable) + reset
+    elif isinstance(variable, int):
+        variable = fg("blue") + str(variable) + reset
+    else:
+        variable = f'{variable!r}'
+
+    fprint(f'fd | {variable_type} | {original_var}: {variable}', nl=nl)
+
+
 def join_path(*args, join_with='\\'):
     return join_with.join(args)
\ No newline at end of file
